/******************************************************************************************
 * Repository: Conhos vscode
 * File name: index.d.ts
 * Author: Sergey Kolmiller
 * Email: <kolserdav@conhos.ru>
 * License: MIT
 * License text: See LICENSE file
 * Copyright: kolserdav, All rights reserved (c)
 * Create Date: Fri Sep 20 2024 13:22:00 GMT+0700 (Krasnoyarsk Standard Time)
 ******************************************************************************************/

export type ServiceTarget = 'common' | 'custom' | 'admin';

export type ServiceTypeCustom = 'node' | 'rust' | 'python' | 'golang' | 'php' | 'ruby';

export type ServiceTypeCommon =
  | 'redis'
  | 'postgres'
  | 'mysql'
  | 'adminer'
  | 'mariadb'
  | 'mongo'
  | 'rabbitmq'
  | 'phpmyadmin'
  | 'pgadmin'
  | 'mongo_express'
  | 'ftp';

export type ServiceTypeCommonPublic =
  | 'adminer'
  | 'phpmyadmin'
  | 'pgadmin'
  | 'mongo_express'
  | 'ftp';

export type ServiceType = ServiceTypeCommon | ServiceTypeCustom;

export type ServiceSize =
  | 'pico'
  | 'nano'
  | 'micro'
  | 'mili'
  | 'santi'
  | 'deci'
  | 'deca'
  | 'hecto'
  | 'kilo';

export type RequiredDependsOn = Partial<Record<ServiceType, { types: ServiceTarget[] }>>;

export type PortType = 'proxy' | 'php';

export type GitUntrackedPolicy = 'checkout' | 'push' | 'merge';

export type Domains = Record<string, string>;

////////////////////////////////////// Config //////////////////////////////////////
/**
 * Do not use the same names in different entities
 * It's necessary to check posicion when check errors
 */
export interface PortStatic {
  location: string;
  path: string;
  index?: string;
}

export interface UpdateConfig {
  parallelism?: number;
  delay?: string;
}

export interface Deploy {
  replicas?: number;
  update_config?: UpdateConfig;
}

export type PortHTTPVersion = '1.0' | '1.1';
export type PortOnOff = 'on' | 'off';

export interface Port<T extends PortType = PortType> {
  port: number;
  type: T;
  location?: T extends 'proxy' ? string : undefined;
  timeout?: T extends 'proxy' ? string : undefined;
  buffer_size?: T extends 'proxy' ? string : undefined;
  proxy_path?: T extends 'proxy' ? string : undefined;
  request_buffering?: T extends 'proxy' ? PortOnOff : undefined;
  buffering?: T extends 'proxy' ? PortOnOff : undefined;
  http_version?: T extends 'proxy' ? PortHTTPVersion : undefined;
  headers?: Record<string, string>;
  static?: T extends 'proxy' ? PortStatic[] : undefined;
  ws?: T extends 'proxy' ? boolean : undefined;
  client_max_body_size?: T extends 'proxy' ? string : undefined;
}

export interface NewDomains {
  serviceName: string;
  domains: Domains;
  serviceType: ServiceType;
  serviceId: string | null;
}

export interface Git {
  url: string;
  branch: string;
  untracked?: GitUntrackedPolicy;
}

export interface Server {
  node_name: string;
  api_key: string;
}

export interface ConfigFile {
  name: string;
  server?: Server;
  services: Record<
    string,
    {
      active: boolean;
      image: ServiceType;
      size: ServiceSize;
      version: string;
      entrypoint?: string[];
      no_restart?: boolean;
      pwd?: string;
      git?: Git;
      exclude?: string[];
      command?: string;
      ports?: Port[];
      volumes?: string[];
      depends_on?: string[];
      domains?: NewDomains['domains'];
      environment?: string[];
      deploy?: Deploy;
    }
  >;
}
////////////////////////////////////////////////////////////////////////////

export type ProxyPaths = Record<string, string>;

export type Volumes = Record<string, string[]>;

export interface DeployData {
  services: {
    type: ServiceType;
    name: string;
    images: string;
    tags: string[];
    hub: string;
  }[];
  sizes: {
    name: string;
    memory: {
      name: string;
      value: number;
    };
    cpus: number;
    storage: string;
    ports: number;
  }[];
  baseValue: number;
  baseCost: number;
}

export type GitType = 'github' | 'gitlab';

export type Status = 'info' | 'warn' | 'error';

export interface UploadFileBody {
  num: number;
  chunk: string | Buffer;
}

export interface ParsedGitRepo {
  user: string;
  project: string;
}

export interface CheckConfigResult {
  msg: string;
  data: string;
  exit: boolean;
  position: {
    lineStart: number;
    lineEnd: number;
    columnStart: number;
    columnEnd: number;
  };
}

export type ConfigFileBackend = Omit<ConfigFile, 'services'> & {
  services: Record<string, ConfigFile['services'][0] & { serviceId: string }>;
};

export interface Config {
  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  value: any;
  detail: string;
  documentation: string;
}

export interface Position {
  line: number;
  character: number;
}

export interface Range {
  start: Position;
  end: Position;
}

export interface CompletionItem {
  label: string;
  kind: CompletionItemKind;
  detail: string;
  documentation: string;
  data: {
    level: number;
    parent: string;
    type: string;
  };
}
